@@ -28,6 +28,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdio.h>
 #include <string.h>
 
+#include <gnuastro/ds9.h>
 #include <gnuastro/wcs.h>
 #include <gnuastro/list.h>
 #include <gnuastro/fits.h>
@@ -191,11 +192,16 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
-      gal_list_str_add(&p->inputs, arg, 0);
-      ++p->numin;
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
+      if(arg[0]!='\0')
+        {
+          gal_list_str_add(&p->inputs, arg, 0);
+          ++p->numin;
+        }
       break;
 
 
@@ -272,6 +278,24 @@ ui_read_check_only_options(struct cropparams *p)
   double *darray;
   int i, checksum;
 
+  /* If it's on the polygon mode and a filename was passed to polygon, then
+     we have to set the mode. It can change the value of '--mode', so this
+     is checked first. */
+  if(p->polygon && p->polygon->status)
+    {
+      switch(p->polygon->status)
+        {
+          case GAL_DS9_COORD_MODE_IMG: p->mode=IMGCROP_MODE_IMG; break;
+          case GAL_DS9_COORD_MODE_WCS: p->mode=IMGCROP_MODE_WCS; break;
+          default:
+            error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+                  "'%s' to fix the problem. The output coordinate mode "
+                  "of 'gal_ds9_reg_read_polygon' (%d) isn't recognized "
+                  "by this function", __func__, PACKAGE_BUGREPORT,
+                  p->polygon->status);
+        }
+    }
+
   /* Make sure that only one of the crop definitions is given. */
   checksum = ( (p->center!=NULL)
                + (p->catname!=NULL)
@@ -280,11 +304,12 @@ ui_read_check_only_options(struct cropparams *p)
   switch(checksum)
     {
     case 0:
-      error(EXIT_FAILURE, 0, "no crop definition. You can use any of the "
-            "following options to define the crop(s): '--center', "
-            "'--catalog', '--section', or '--polygon'. Please run this "
-            "command for more information:\n\n"
-            "    $ info gnuastro \"Crop modes\"\n");
+      if(p->cp.printparams==0)
+        error(EXIT_FAILURE, 0, "no crop definition. You can use any of "
+              "the following options to define the crop(s): '--center', "
+              "'--catalog', '--section', or '--polygon'. Please run this "
+              "command for more information:\n\n"
+              "    $ info gnuastro \"Crop modes\"\n");
     case 1:
       /* Everything is ok, just ignore the switch structure. */
       break;
@@ -350,7 +375,7 @@ ui_read_check_only_options(struct cropparams *p)
               "best for this option to have a value", p->catname);
 
       /* If it is a FITS file, we need the HDU. */
-      if( gal_fits_name_is_fits(p->catname) && p->cathdu==NULL )
+      if( gal_fits_file_recognized(p->catname) && p->cathdu==NULL )
         error(EXIT_FAILURE, 0, "%s: no hdu given. Please use the '--cathdu' "
               "option to specify which extension contains the table",
               p->catname);
@@ -875,9 +900,17 @@ ui_preparations_to_img_mode(struct cropparams *p)
           "will get wrong results. You can suppress this warning "
           "with '--quiet'", p->inputs->v, p->cp.hdu);
 
-  /* Convert the given coordinates. */
+  /* Convert the given width into pixels. */
   if(p->center || p->catname)
     {
+      /* This function may be called before the formal sanity checks are
+         complete. So we need to check for the presence of '--width' again
+         here. */
+      if(p->width==NULL)
+        error(EXIT_FAILURE, 0, "no crop width specified. When crops are "
+              "defined by their center (with '--center' or '--catalog') a "
+              "width is necessary (using the '--width' option)");
+
       /* Check the requested width and convert it to pixels. */
       darr=p->width->array;
       if(wcs->naxis<p->width->size)
@@ -901,11 +934,10 @@ ui_preparations_to_img_mode(struct cropparams *p)
 static void
 ui_preparations_to_img_mode_values(struct cropparams *p)
 {
-  struct wcsprm *wcs=p->imgs[0].wcs;
-
   int i, status, *stat=NULL;
   gal_data_t *tmp, *coords=NULL;
-  size_t ndim=wcs->naxis, plysize=p->nvertices;
+  struct wcsprm *wcs=p->imgs[0].wcs;
+  size_t ndim=p->imgs[0].ndim, plysize=p->nvertices;
   double *phi=NULL, *theta=NULL, *pixcrd=NULL, *imgcrd=NULL;
 
   /* When central coordinates (either from '--center' or '--catalog' are
@@ -970,6 +1002,8 @@ ui_preparations_to_img_mode_values(struct cropparams *p)
 
 
 
+
+
 void
 ui_preparations(struct cropparams *p)
 {
@@ -988,6 +1022,7 @@ ui_preparations(struct cropparams *p)
       ui_preparations_to_img_mode(p);
     }
 
+
   /* For polygon and section, there should be no center checking. */
   if(p->polygon || p->section)
     p->checkcenter=0;
@@ -1022,13 +1057,7 @@ ui_preparations(struct cropparams *p)
       img->wcs=gal_wcs_read_fitsptr(tmpfits, p->cp.wcslinearmatrix,
                                     p->hstartwcs, p->hendwcs, &img->nwcs);
       if(img->wcs)
-        {
-          gal_wcs_decompose_pc_cdelt(img->wcs);
-          status=wcshdo(0, img->wcs, &img->nwcskeys, &img->wcstxt);
-          if(status)
-            error(EXIT_FAILURE, 0, "wcshdo ERROR %d: %s", status,
-                  wcs_errmsg[status]);
-        }
+        img->wcstxt=gal_wcs_write_wcsstr(img->wcs, &img->nwcskeys);
       else
         if(p->mode==IMGCROP_MODE_WCS)
           error(EXIT_FAILURE, 0, "%s (hdu %s): the WCS structure is "
