@@ -5,7 +5,7 @@ MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,23 +31,24 @@ struct mkonthread
 {
   /* General parameters: */
   double                r;   /* Elliptical radius at this point.      */
-  double         coord[2];   /* Pixel coordinate.                     */
-  double         lower[2];   /* Coordinates of lower pixel position.  */
-  double        higher[2];   /* Coordinates of higher pixel position. */
-  double             c[1];   /* Cosine of position angle(s).          */
-  double             s[1];   /* Sine of position angle(s).            */
-  double             q[1];   /* Axis ratio(s).                        */
-  double        center[2];   /* Center (in FITS) in oversampled image.*/
+  double         coord[3];   /* Pixel coord. (from profile center).   */
+  double         lower[3];   /* Coordinates of lower pixel position.  */
+  double        higher[3];   /* Coordinates of higher pixel position. */
+  double             c[3];   /* Cosine of position angle(s).          */
+  double             s[3];   /* Sine of position angle(s).            */
+  double             q[2];   /* Axis ratio(s).                        */
+  double        center[3];   /* Center (in FITS) in oversampled image.*/
   double (*profile)(struct mkonthread *); /* Function to use.         */
   double           truncr;   /* Truncation radius in pixels.          */
   double         intruncr;   /* Inner truncation radius in pixels.    */
-  long           width[2];   /* Enclosing box in FITS axes, not C.    */
+  long           width[3];   /* Enclosing box in FITS axes, not C.    */
   float          peakflux;   /* Flux at profile peak.                 */
   float        brightness;   /* The brightness of the profile.        */
-  uint8_t            func;   /* Radial function of the profile.       */
+  uint8_t            func;   /* Radial function code of the profile.  */
   long            *onaxes;   /* Sides of the unover-sampled image.    */
-  long        fpixel_i[2];   /* fpixel_i before running overlap.      */
+  long        fpixel_i[3];   /* fpixel_i before running overlap.      */
   int          correction;   /* ==1: correct the pixels afterwards.   */
+  unsigned long  rng_seed;   /* Seed used to generate this profile.   */
 
   /* Random number generator: */
   gsl_rng            *rng;   /* Copy of main random number generator. */
