@@ -124,6 +124,9 @@ ui_profile_name_read(char *string, size_t row)
   else if ( !strcmp("distance", string) )
     return PROFILE_DISTANCE;
 
+  else if ( !strcmp("azimuth", string) )
+    return PROFILE_DISTANCE;
+
   else if ( !strcmp("custom", string) )
     return PROFILE_CUSTOM;
 
@@ -159,6 +162,7 @@ ui_profile_name_write(int profile_code)
     case PROFILE_FLAT:           return "flat";
     case PROFILE_CIRCUMFERENCE:  return "circum";
     case PROFILE_DISTANCE:       return "distance";
+    case PROFILE_AZIMUTH:        return "azimuth";
     default:
       error(EXIT_FAILURE, 0, "%s: %d not recognized as a profile code",
             __func__, profile_code);
@@ -256,16 +260,17 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->catname)
         argp_error(state, "only one argument (input catalog) may be given");
       else
-        p->catname=arg;
+        if(arg[0]!='\0') p->catname=arg;
       break;
 
-
     /* This is an option, set its value. */
     default:
       return gal_options_set_from_key(key, arg, p->cp.poptions, &p->cp);
@@ -427,6 +432,7 @@ ui_parse_kernel(struct argp_option *option, char *arg,
         case PROFILE_FLAT:          need = kernel->flag==2 ? 1 : 2;  break;
         case PROFILE_CIRCUMFERENCE: need = kernel->flag==2 ? 1 : 2;  break;
         case PROFILE_DISTANCE:      need = kernel->flag==2 ? 1 : 2;  break;
+        case PROFILE_AZIMUTH:       need = kernel->flag==2 ? 1 : 2;  break;
         default:
           error_at_line(EXIT_FAILURE, 0, filename, lineno, "%s: a bug! "
                         "Please contact us at %s to correct the issue. "
@@ -572,13 +578,13 @@ ui_check_options_and_arguments(struct mkprofparams *p)
               "catalog ('%s'). The parameters necessary to build a single "
               "kernel output should be given to '--kernel', not in a "
               "catalog", p->catname);
-      p->catname="kernel.option";
+      p->catname="kernel.optional";
     }
   else
     {
       if(p->catname)
         {
-          if( gal_fits_name_is_fits(p->catname) && p->cp.hdu==NULL)
+          if( gal_fits_file_recognized(p->catname) && p->cp.hdu==NULL)
             error(EXIT_FAILURE, 0, "no 'hdu' specified for the input FITS "
                   "table '%s', to ", p->catname);
         }
@@ -606,15 +612,18 @@ ui_check_options_and_arguments(struct mkprofparams *p)
       tmpname=gal_checkset_automatic_output(&p->cp,
                                             ( p->catname
                                               ? p->catname
-                                              : "makeprofiles" ), ".fits");
+                                              : "makeprofiles" ),
+                                            ( p->kernel
+                                              ? ".fits"
+                                              : "_profiles.fits" ));
       p->mergedimgname=gal_checkset_malloc_cat(p->outdir, tmpname);
       free(tmpname);
     }
   p->basename=gal_checkset_not_dir_part(p->mergedimgname);
 
 
-  /* If a merged image is requested (or '--kernel' the option is called),
-     then delete the final filename if it exists. */
+  /* If a merged image is requested (or '--kernel' is called), then delete
+     the final filename if it exists. */
   if(p->nomerged==0 && p->kernel)
     gal_checkset_writable_remove(p->mergedimgname, p->cp.keep,
                                  p->cp.dontdelete);
@@ -970,7 +979,9 @@ ui_read_cols_3d(struct mkprofparams *p)
               corrtype=gal_data_copy_to_new_type_free(tmp, GAL_TYPE_UINT8);
               p->f=corrtype->array;
 
-              /* Check if they are in the correct range. */
+              /* Check if they are in the correct range. For profile names
+                 given as string, a non-matching string will result in an
+                 error, so there is no need for this in that scenario. */
               for(i=0;i<p->num;++i)
                 if(p->f[i]<=PROFILE_INVALID || p->f[i]>=PROFILE_MAXIMUM_CODE)
                   error(EXIT_FAILURE, 0, "%s: row %zu, the function "
@@ -984,6 +995,16 @@ ui_read_cols_3d(struct mkprofparams *p)
                         "    $ info %s\n", p->catname, i+1, p->f[i],
                         PROFILE_INVALID, PROFILE_MAXIMUM_CODE, PROGRAM_EXEC);
             }
+
+          /* General profile sanity checks. */
+          for(i=0;i<p->num;++i)
+            {
+              /* Azimuthal profile not yet supported for ellipsoids. */
+              if(p->f[i]==PROFILE_AZIMUTH)
+                error(EXIT_FAILURE, 0, "%s: row %zu: the azimuthal "
+                      "angle profile is not yet supported for 3D "
+                      "datasets", p->catname, i+1);
+            }
           break;
 
         case 5:
@@ -1373,9 +1394,9 @@ static void
 ui_prepare_canvas(struct mkprofparams *p)
 {
   float *f, *ff;
+  int setshift=0;
   long width[3]={1,1,1};
   size_t tndim, *tdsize;
-  int status=0, setshift=0;
   double truncr, semiaxes[3], euler_deg[3];
   size_t i, nshift=0, *dsize=NULL, ndim_counter;
 
@@ -1527,19 +1548,6 @@ ui_prepare_canvas(struct mkprofparams *p)
       if(p->out->unit==NULL)
         gal_checkset_allocate_copy("Brightness", &p->out->unit);
     }
-
-
-  /* When individual mode is requested, write the WCS structure to a header
-     string to speed up the process: if we don't do it here, this process
-     will be necessary on every individual profile's output. So it is much
-     more efficient done once here. */
-  if(p->individual && p->wcs)
-    {
-      status=wcshdo(WCSHDO_safe, p->wcs, &p->wcsnkeyrec, &p->wcsheader);
-      if(status)
-        error(EXIT_FAILURE, 0, "wcshdo error %d: %s", status,
-              wcs_errmsg[status]);
-    }
 }
 
 
@@ -1858,11 +1866,22 @@ ui_preparations(struct mkprofparams *p)
   else
     ui_prepare_canvas(p);
 
-  /* Read the (possible) RA/Dec inputs into X and Y for the builder.  NOTE:
-     It may happen that there are no input columns, in that case, just
-     ignore this step.*/
-  if(p->wcs && p->num)
-    ui_finalize_coordinates(p);
+  /* Preparations now that we have WCS (if any was given in any way: either
+     from a background or from options).  */
+  if(p->wcs)
+    {
+      /* Read the (possible) RA/Dec inputs into X and Y for the builder.
+         NOTE: It may happen that there are no input columns, in that case,
+         just ignore this step.*/
+      if(p->num)
+        ui_finalize_coordinates(p);
+
+      /* If individual mode is activated, write the WCS as a string here
+         (earlier than needed). This is because it will be necessary for
+         every individual profile, but it will identical (except for the
+         'CRPIX's that will be changed). */
+      p->wcsstr=gal_wcs_write_wcsstr(p->wcs, &p->wcsnkeyrec);
+    }
 
   /* Prepare the random number generator. */
   p->rng=gal_checkset_gsl_rng(p->envseed, &p->rng_name, &p->rng_seed);
@@ -2063,8 +2082,7 @@ ui_free_report(struct mkprofparams *p, struct timeval *t1)
     }
 
   /* Free the WCS headers string that was defined for individual mode. */
-  if(p->individual)
-    free(p->wcsheader);
+  if(p->wcsstr) free(p->wcsstr);
 
   /* Free the random number generator: */
   gsl_rng_free(p->rng);
