@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -61,7 +61,7 @@ int
 gal_jpeg_suffix_is_jpeg(char *name);
 
 gal_data_t *
-gal_jpeg_read(char *filename, size_t minmapsize);
+gal_jpeg_read(char *filename, size_t minmapsize, int quietmmap);
 
 void
 gal_jpeg_write(gal_data_t *in, char *filename, uint8_t quality,
