@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -52,6 +52,9 @@ gal_list_str_add(gal_list_str_t **list, char *value,
 {
   gal_list_str_t *newnode;
 
+  /* If the value is a NULL pointer, don't add to the list. */
+  if(value==NULL) return;
+
   errno=0;
   newnode=malloc(sizeof *newnode);
   if(newnode==NULL)
@@ -102,6 +105,21 @@ gal_list_str_number(gal_list_str_t *list)
 
 
 
+gal_list_str_t *
+gal_list_str_last(gal_list_str_t *list)
+{
+  if(list)
+    {
+      while(list->next!=NULL) list=list->next;
+      return list;
+    }
+  else return NULL;
+}
+
+
+
+
+
 void
 gal_list_str_print(gal_list_str_t *list)
 {
@@ -225,6 +243,21 @@ gal_list_i32_number(gal_list_i32_t *list)
 
 
 
+gal_list_i32_t *
+gal_list_i32_last(gal_list_i32_t *list)
+{
+  if(list)
+    {
+      while(list->next!=NULL) list=list->next;
+      return list;
+    }
+  else return NULL;
+}
+
+
+
+
+
 void
 gal_list_i32_print(gal_list_i32_t *list)
 {
@@ -374,6 +407,22 @@ gal_list_sizet_number(gal_list_sizet_t *list)
 
 
 
+
+gal_list_sizet_t *
+gal_list_sizet_last(gal_list_sizet_t *list)
+{
+  if(list)
+    {
+      while(list->next!=NULL) list=list->next;
+      return list;
+    }
+  else return NULL;
+}
+
+
+
+
+
 void
 gal_list_sizet_print(gal_list_sizet_t *list)
 {
@@ -524,6 +573,21 @@ gal_list_f32_number(gal_list_f32_t *list)
 
 
 
+gal_list_f32_t *
+gal_list_f32_last(gal_list_f32_t *list)
+{
+  if(list)
+    {
+      while(list->next!=NULL) list=list->next;
+      return list;
+    }
+  else return NULL;
+}
+
+
+
+
+
 void
 gal_list_f32_reverse(gal_list_f32_t **list)
 {
@@ -681,6 +745,21 @@ gal_list_f64_number(gal_list_f64_t *list)
 
 
 
+gal_list_f64_t *
+gal_list_f64_last(gal_list_f64_t *list)
+{
+  if(list)
+    {
+      while(list->next!=NULL) list=list->next;
+      return list;
+    }
+  else return NULL;
+}
+
+
+
+
+
 void
 gal_list_f64_print(gal_list_f64_t *list)
 {
@@ -836,6 +915,22 @@ gal_list_void_number(gal_list_void_t *list)
 
 
 
+
+gal_list_void_t *
+gal_list_void_last(gal_list_void_t *list)
+{
+  if(list)
+    {
+      while(list->next!=NULL) list=list->next;
+      return list;
+    }
+  else return NULL;
+}
+
+
+
+
+
 void
 gal_list_void_reverse(gal_list_void_t **list)
 {
@@ -1087,11 +1182,11 @@ gal_list_dosizet_pop_smallest(gal_list_dosizet_t **largest,
     }
   else
     {
-      /* If `smallest' is NULL, `largest' should also be NULL. */
+      /* If 'smallest' is NULL, 'largest' should also be NULL. */
       if(*largest)
-        error(EXIT_FAILURE, 0, "%s: `largest' and `smallest' pointers must "
+        error(EXIT_FAILURE, 0, "%s: 'largest' and 'smallest' pointers must "
               "both be non-NULL or both be NULL. However, in this call, "
-              "`smallest' was NULL while `largest' isn't NULL", __func__);
+              "'smallest' was NULL while 'largest' isn't NULL", __func__);
       value=GAL_BLANK_SIZE_T;
       *tosort=NAN;
     }
@@ -1194,7 +1289,7 @@ gal_list_data_add(gal_data_t **list, gal_data_t *newnode)
       toadd=tmp;
     }
   else
-    /* Its not a list, so just set it to `toadd'. */
+    /* Its not a list, so just set it to 'toadd'. */
     toadd=newnode;
 
 
@@ -1210,14 +1305,14 @@ gal_list_data_add(gal_data_t **list, gal_data_t *newnode)
 void
 gal_list_data_add_alloc(gal_data_t **list, void *array, uint8_t type,
                         size_t ndim, size_t *dsize, struct wcsprm *wcs,
-                        int clear, size_t minmapsize, char *name,
-                        char *unit, char *comment)
+                        int clear, size_t minmapsize, int quietmmap,
+                        char *name, char *unit, char *comment)
 {
   gal_data_t *newnode;
 
   /* Put all the input information into a new data structure node. */
   newnode=gal_data_alloc(array, type, ndim, dsize, wcs, clear,
-                         minmapsize, name, unit, comment);
+                         minmapsize, quietmmap, name, unit, comment);
 
   /* Add the new node to the list. */
   gal_list_data_add(list, newnode);
@@ -1273,6 +1368,32 @@ gal_list_data_reverse(gal_data_t **list)
 
 
 
+gal_data_t **
+gal_list_data_to_array_ptr(gal_data_t *list, size_t *num)
+{
+  size_t i, n;
+  gal_data_t *tmp, **out;
+
+  /* Count how many columns are necessary. */
+  n=*num=gal_list_data_number(list);
+
+  /* Allocate space for the array. */
+  errno=0;
+  out=malloc(n * sizeof *out);
+  if(out==NULL)
+    error(EXIT_FAILURE, 0, "%s: couldn't allocate %zu bytes", __func__,
+          n * sizeof *out);
+
+  /* Fill up the array with the pointers and return. */
+  i=0;
+  for(tmp=list;tmp!=NULL;tmp=tmp->next) out[i++]=tmp;
+  return out;
+}
+
+
+
+
+
 size_t
 gal_list_data_number(gal_data_t *list)
 {
@@ -1288,6 +1409,21 @@ gal_list_data_number(gal_data_t *list)
 
 
 
+
+gal_data_t *
+gal_list_data_last(gal_data_t *list)
+{
+  if(list)
+    {
+      while(list->next!=NULL) list=list->next;
+      return list;
+    }
+  else return NULL;
+}
+
+
+
+
 void
 gal_list_data_free(gal_data_t *list)
 {
